@@ -2,7 +2,6 @@
 
 ;;; Code:
 
-(custom-add-loads 'xml '("xml"))
 (custom-add-loads 'url '("url-gw" "url-irc" "url-news" "url-vars" "url"))
 (custom-add-loads 'ssl '("ssl"))
 (custom-add-loads 'url-cookie '("url-cookie" "url-vars"))
@@ -22,7 +21,7 @@
 (custom-add-loads 'faces '("font"))
 (custom-add-loads 'processes '("socks"))
 (custom-add-loads 'w3-hooks '("w3-cus"))
-(custom-add-loads 'w3 '("url-vars" "w3-cus" "w3-script" "xml"))
+(custom-add-loads 'w3 '("url-vars" "w3-cus" "w3-script"))
 (custom-add-loads 'url-file '("url-cache" "url-vars"))
 (custom-add-loads 'url-news '("url-news"))
 (custom-add-loads 'w3-display '("w3-cus"))
