@@ -1,14 +1,14 @@
 ;;; url-vars.el --- Variables for Uniform Resource Locator tool
 ;; Author: wmperry
-;; Created: 1998/06/09 14:54:28
-;; Version: 1.80
+;; Created: 1998/12/18 02:19:58
+;; Version: 1.1.1.2
 ;; Keywords: comm, data, processes, hypermedia
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Copyright (c) 1993 - 1996 by William M. Perry <wmperry@cs.indiana.edu>
-;;; Copyright (c) 1996 - 1998 Free Software Foundation, Inc.
+;;; Copyright (c) 1996 - 1999 Free Software Foundation, Inc.
 ;;;
-;;; This file is not part of GNU Emacs, but the same permissions apply.
+;;; This file is part of GNU Emacs.
 ;;;
 ;;; GNU Emacs is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -38,7 +38,7 @@
     (defmacro defcustom (var value doc &rest args) 
       (` (defvar (, var) (, value) (, doc))))))
 
-(defconst url-version (let ((x "p4.0pre.23"))
+(defconst url-version (let ((x "p4.0pre.39"))
 			(if (string-match "State: \\([^ \t\n]+\\)" x)
 			    (substring x (match-beginning 1) (match-end 1))
 			  x))
@@ -103,6 +103,7 @@ each item in the list will be an argument to the url-current-callback-func.")
 				      url-current-mime-encoding
 				      url-current-mime-headers
 				      url-current-mime-type
+				      url-current-mime-charset
 				      url-current-mime-viewer
 				      url-current-object
 				      url-current-referer
@@ -368,6 +369,7 @@ buffers when used for multiple requests, cf. `url-multiple-p'")
 (defvar url-current-mime-encoding nil "MIME encoding of current document.")
 (defvar url-current-mime-headers nil "An alist of MIME headers.")
 (defvar url-current-mime-type nil "MIME type of current document.")
+(defvar url-current-mime-charset nil "MIME charset of current document.")
 (defvar url-current-mime-viewer nil "How to view the current MIME doc.")
 (defvar url-current-passwd-count 0 "How many times password has failed.")
 (defvar url-gopher-types "0123456789+gIThws:;<"
@@ -400,6 +402,10 @@ while large downloads occur."
   :group 'url)
 (make-variable-buffer-local 'url-be-asynchronous)
 
+(defvar url-mime-content-type-charset-regexp
+  ";[ \t]*charset=\"?\\([^\"]+\\)\"?"
+  "Regexp used in parsing `Content-Type' for a charset indication")
+
 (defvar url-request-data nil "Any data to send with the next request.")
 
 (defvar url-request-extra-headers nil
@@ -593,6 +599,9 @@ Each hook is called with a single argument URL and should return a tranformed
 url with a valid scheme (e.g., \"gnu\" --> \"http://www.gnu.org/\"), or nil
 otherwise.")
 
+(defvar url-load-hook nil
+  "*Hooks to be run after initalizing the URL library.")
+
 ;;; Make OS/2 happy - yeeks
 (defvar	tcp-binary-process-input-services nil
   "*Make OS/2 happy with our CRLF pairs...")
