@@ -1,20 +1,20 @@
 ;;; url.el --- Uniform Resource Locator retrieval tool
 ;; Author: wmperry
-;; Created: 1998/05/25 01:54:19
-;; Version: 1.90
+;; Created: 1998/12/30 11:26:49
+;; Version: 1.9
 ;; Keywords: comm, data, processes, hypermedia
 
 ;;; LCD Archive Entry:
 ;;; url|William M. Perry|wmperry@cs.indiana.edu|
 ;;; Functions for retrieving/manipulating URLs|
-;;; 1998/05/25 01:54:19|1.90|Location Undetermined
+;;; 1998/12/30 11:26:49|1.9|Location Undetermined
 ;;;
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Copyright (c) 1993 - 1996 by William M. Perry <wmperry@cs.indiana.edu>
-;;; Copyright (c) 1996 - 1998 Free Software Foundation, Inc.
+;;; Copyright (c) 1996 - 1999 Free Software Foundation, Inc.
 ;;;
-;;; This file is not part of GNU Emacs, but the same permissions apply.
+;;; This file is part of GNU Emacs.
 ;;;
 ;;; GNU Emacs is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -102,6 +102,10 @@
 (autoload 'url-netrek "url-misc")
 (autoload 'url-news "url-news")
 (autoload 'url-nntp "url-news")
+(autoload 'url-cid "url-cid")
+
+(if (featurep 'ldap)
+    (autoload 'url-ldap "url-ldap"))
 
 (autoload 'url-open-stream "url-gw")
 (autoload 'url-mime-response-p "url-http")
@@ -577,9 +581,12 @@ If file does not exist, returns nil."
 	  ((member type '("ftp" "file"))
 	   (let ((fname (if (url-host urlobj)
 			    (concat "/"
-				    (if (url-user urlobj)
-					(concat (url-user urlobj) "@")
-				      "")
+				    (cond
+				     ((url-user urlobj)
+				      (concat (url-user urlobj) "@"))
+				     ((member type '("ftp"))
+				      "anonymous@")
+				     (t ""))
 				    (url-host urlobj) ":"
 				    (url-filename urlobj))
 			  (url-filename urlobj))))
@@ -655,7 +662,7 @@ accessible."
 		       (concat "/"
 			       (if (url-user urlobj)
 				   (concat (url-user urlobj) "@")
-				 "")
+				 "anonymous@")
 			       (url-host urlobj) ":"
 			       (url-filename urlobj))
 		     (url-filename urlobj))))
@@ -715,7 +722,7 @@ numbers, etc."
 		       (concat "/"
 			       (if (url-user urlobj)
 				   (concat (url-user urlobj) "@")
-				 "")
+				 "anonymous@")
 			       (url-host urlobj) ":"
 			       (url-filename urlobj))
 		     (url-filename urlobj))))
@@ -917,10 +924,13 @@ dumped with emacs."
     (url-register-protocol 'irc        nil 'url-identity-expander "6667")
     (url-register-protocol 'data       nil 'url-identity-expander)
     (url-register-protocol 'netrek     nil 'url-identity-expander)
+    (url-register-protocol 'cid        nil 'url-identity-expander)
     (url-register-protocol 'rlogin)
     (url-register-protocol 'telnet)
     (url-register-protocol 'tn3270)
     (url-register-protocol 'proxy)
+    (if (featurep 'ldap)
+	(url-register-protocol 'ldap))
     (url-register-protocol 'auto 'url-handle-no-scheme)
 
     ;; Register all the authentication schemes we can handle
@@ -932,7 +942,7 @@ dumped with emacs."
 
     (setq url-cookie-file
 	  (or url-cookie-file
-	      (expand-file-name "~/.w3/cookies")))
+	      (expand-file-name "cookies" w3-configuration-directory)))
     
     (setq url-global-history-file
 	  (or url-global-history-file
@@ -941,7 +951,7 @@ dumped with emacs."
 	      (and (memq system-type '(axp-vms vax-vms))
 		   (expand-file-name "~/mosaic.global-history"))
 	      (condition-case ()
-		  (expand-file-name "~/.w3/history")
+		  (expand-file-name (concat w3-configuration-directory "history"))
 		(error nil))))
   
     ;; Parse the global history file if it exists, so that it can be used
@@ -1030,8 +1040,10 @@ dumped with emacs."
     (run-hooks 'url-load-hook)
     (setq url-setup-done t)))
 
+(eval-and-compile
 (defvar url-get-url-filename-chars "-%.?@a-zA-Z0-9()_/:~=&"
   "Valid characters in a URL")
+)
 
 ;;;###autoload
 (defun url-get-url-at-point (&optional pt)
@@ -1123,59 +1135,71 @@ Second arg DEFAULT is a URL to start with if URL is relative.
 If DEFAULT is nil or missing, the current buffer's URL is used.
 Path components that are `.' are removed, and 
 path components followed by `..' are removed, along with the `..' itself."
-  (if url
+  (if (and url (not (string-match "^#" url)))
       (setq url (mapconcat (function (lambda (x)
 				       (if (memq x '(?  ?\n ?\r))
 					   ""
 					 (char-to-string x))))
 			   (url-strip-leading-spaces
 			    (url-eat-trailing-space url)) "")))
+  (setq default (cond
+		 ((vectorp default) default)
+		 (default (url-generic-parse-url default))
+		 (url-current-object url-current-object)
+		 (t (url-generic-parse-url (url-view-url t)))))
+
   (cond
-   ((null url) nil)			; Something hosed!  Be graceful
+   ((= (length url) 0)			; nil or empty string
+    (url-recreate-url default))
    ((string-match "^#" url)		; Offset link, use it raw
     url)
    (t
     (let* ((urlobj (url-generic-parse-url url))
 	   (inhibit-file-name-handlers t)
-	   (defobj (cond
-		    ((vectorp default) default)
-		    (default (url-generic-parse-url default))
-		    (url-current-object url-current-object)
-		    (t (url-generic-parse-url (url-view-url t)))))
 	   (expander (cdr-safe
 		      (cdr-safe
 		       (assoc (or (url-type urlobj)
-				  (url-type defobj))
+				  (url-type default))
 			      url-registered-protocols)))))
       (if (string-match "^//" url)
-	  (setq urlobj (url-generic-parse-url (concat (url-type defobj) ":"
+	  (setq urlobj (url-generic-parse-url (concat (url-type default) ":"
 						      url))))
       (if (fboundp expander)
-	  (funcall expander urlobj defobj)
+	  (funcall expander urlobj default)
 	(message "Unknown URL scheme: %s" (or (url-type urlobj)
-					     (url-type defobj)))
-	(url-identity-expander urlobj defobj))
+					     (url-type default)))
+	(url-identity-expander urlobj default))
       (url-recreate-url urlobj)))))
 
 (defun url-default-expander (urlobj defobj)
   ;; The default expansion routine - urlobj is modified by side effect!
-  (url-set-type urlobj (or (url-type urlobj) (url-type defobj)))
-  (url-set-port urlobj (or (url-port urlobj)
-			   (and (string= (url-type urlobj)
-					 (url-type defobj))
-				(url-port defobj))))
-  (if (not (string= "file" (url-type urlobj)))
-      (url-set-host urlobj (or (url-host urlobj) (url-host defobj))))
-  (if (string= "ftp"  (url-type urlobj))
-      (url-set-user urlobj (or (url-user urlobj) (url-user defobj))))
-  (if (string= (url-filename urlobj) "")
-      (url-set-filename urlobj "/"))
-  (if (string-match "^/" (url-filename urlobj))
+  (if (url-type urlobj)
+      ;; Well, they told us the scheme, let's just go with it.
       nil
-    (url-set-filename urlobj
-		      (url-remove-relative-links
-		       (concat (url-basepath (url-filename defobj))
-			       (url-filename urlobj))))))
+    (url-set-type urlobj (or (url-type urlobj) (url-type defobj)))
+    (url-set-port urlobj (or (url-port urlobj)
+			     (and (string= (url-type urlobj)
+					   (url-type defobj))
+				  (url-port defobj))))
+    (if (not (string= "file" (url-type urlobj)))
+	(url-set-host urlobj (or (url-host urlobj) (url-host defobj))))
+    (if (string= "ftp"  (url-type urlobj))
+	(url-set-user urlobj (or (url-user urlobj) (url-user defobj))))
+    (if (string= (url-filename urlobj) "")
+	(url-set-filename urlobj "/"))
+    (if (string-match "^/" (url-filename urlobj))
+	nil
+      (let ((query nil)
+	    (file nil)
+	    (sepchar nil))
+	(if (string-match "[?#]" (url-filename urlobj))
+	    (setq query (substring (url-filename urlobj) (match-end 0))
+		  file (substring (url-filename urlobj) 0 (match-beginning 0))
+		  sepchar (substring (url-filename urlobj) (match-beginning 0) (match-end 0)))
+	  (setq file (url-filename urlobj)))
+	(setq file (url-remove-relative-links
+		    (concat (url-basepath (url-filename defobj)) file)))
+	(url-set-filename urlobj (if query (concat file sepchar query) file))))))
 
 (defun url-identity-expander (urlobj defobj)
   (url-set-type urlobj (or (url-type urlobj) (url-type defobj))))
@@ -1402,12 +1426,25 @@ forbidden in URL encoding."
   (let ((new (if (not (string-match "^/" name))
 		 (concat "/" name)
 	       name)))
+
+    ;; If it ends with a '/.' or '/..', tack on a trailing '/' sot hat
+    ;; the tests that follow are not too complicated in terms of
+    ;; looking for '..' or '../', etc.
+    (if (string-match "/\\.+$" new)
+	(setq new (concat new "/")))
+
+    ;; Remove '/./' first
     (while (string-match "/\\(\\./\\)" new)
       (setq new (concat (substring new 0 (match-beginning 1))
 			(substring new (match-end 1)))))
+
+    ;; Then remove '/../'
     (while (string-match "/\\([^/]*/\\.\\./\\)" new)
       (setq new (concat (substring new 0 (match-beginning 1))
 			(substring new (match-end 1)))))
+
+    ;; Remove cruft at the beginning of the string, so people that put
+    ;; in extraneous '..' because they are morons won't lose.
     (while (string-match "^/\\.\\.\\(/\\)" new)
       (setq new (substring new (match-beginning 1) nil)))
     new))
@@ -1928,6 +1965,13 @@ retrieve a URL by its HTML source."
       (url-warn 'url (format "Unknown proxy directive: %s" proxy) 'critical)
       nil))))
 
+(defun url-looks-like-html (&optional buf)
+  (save-excursion
+    (set-buffer (or buf (current-buffer)))
+    (goto-char (point-min))
+    (let ((case-fold-search t))
+      (re-search-forward "<\\(html\\|head\\|title\\|body\\).*>" nil t))))
+
 (defun url-retrieve-internally (url &optional no-cache)
   (let* ((url-working-buffer (if (and url-multiple-p
 				      (string-equal
@@ -1984,14 +2028,20 @@ retrieve a URL by its HTML source."
      ((and (not url-inhibit-mime-parsing)
 	   (or cached (url-mime-response-p t)))
       (or cached (url-parse-mime-headers nil t))))
+
     (if (and (or (not url-be-asynchronous)
 		 (not (equal type "http")))
 	     url-current-object
 	     (not url-current-mime-type))
-	(setq url-current-mime-type (mm-extension-to-mime
-				     (url-file-extension
-				      (url-filename
-				       url-current-object)))))
+	(progn
+	  (setq url-current-mime-type (mm-extension-to-mime
+				       (url-file-extension
+					(url-filename
+					 url-current-object))))
+	  (if (and (not url-current-mime-type)
+		   (url-looks-like-html))
+	      (setq url-current-mime-type "text/html"))))
+
     (if (not url-be-asynchronous)
 	(url-store-in-cache url-working-buffer))
     (if (not url-global-history-hash-table)
