@@ -1,12 +1,12 @@
 ;;; w3-display.el --- display engine
 ;; Author: wmperry
-;; Created: 1998/06/22 23:36:29
-;; Version: 1.222
+;; Created: 1998/12/28 21:44:07
+;; Version: 1.8
 ;; Keywords: faces, help, hypermedia
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Copyright (c) 1996 by William M. Perry <wmperry@cs.indiana.edu>
-;;; Copyright (c) 1996 - 1998 Free Software Foundation, Inc.
+;;; Copyright (c) 1996 - 1999 Free Software Foundation, Inc.
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; This file is part of GNU Emacs.
@@ -113,7 +113,8 @@
        (w3-get-face-info background-color bgcolor)
        (setq w3-face-font-spec (make-font
 				:weight (car w3-face-font-weight)
-				:family (car w3-face-font-family)
+				:family  (if (not w3-user-fonts-take-precedence)
+					     (car w3-face-font-family))
 				:size (car w3-face-font-size))))))
 
   (defmacro w3-pop-all-face-info ()
@@ -164,7 +165,8 @@
      (w3-running-xemacs
       (if (and (not (sit-for 0)) (input-pending-p))
 	  (condition-case ()
-	      (dispatch-event (next-command-event))
+	      (let ((buffer-read-only t))
+		(dispatch-event (next-command-event)))
 	    (error nil))))
      (t
       (if (and (not (sit-for 0)) (input-pending-p))
@@ -250,6 +252,71 @@
 	  (setq voice nil))
 	(or voice (car w3-active-voices)))))
 
+(defun w3-display-colors-too-close-p (foreground background)
+  ;; This algorithm could be _MUCH_ better. :)
+  ;;
+  ;; Perhaps doing something along the lines of converting them both
+  ;; to grayscale values, then considering them worthless if they are
+  ;; within some user-configurable delta?
+  (equal foreground background))
+
+(defun w3-get-default-color (backgroundp)
+  (if w3-running-xemacs
+      (or (if backgroundp
+	      (face-background-name 'default)
+	    (face-foreground-name 'default))
+	  (frame-property nil (if backgroundp 'background-color 'foreground-color)))
+      (or (if backgroundp
+	      (face-background 'default)
+	    (face-foreground 'default))
+	  (frame-parameter nil (if backgroundp 'background-color 'foreground-color)))))
+
+(defun w3-display-background-useless-p (color)
+  (let ((foreground-color (font-color-rgb-components
+			   (w3-get-default-color nil)))
+	(background-color (font-color-rgb-components color)))
+    (w3-display-colors-too-close-p foreground-color background-color)))
+
+(defun w3-display-foreground-useless-p (color)
+  (let ((background-color (font-color-rgb-components
+			   (w3-get-default-color t)))
+	(foreground-color (font-color-rgb-components color)))
+    (w3-display-colors-too-close-p foreground-color background-color)))
+
+(defun w3-display-infer-contrasting-color (color)
+  ;; Simple little algorithm suggested by Adam Hammer
+  ;; <hammer@math.purdue.edu>
+  ;; 
+  ;; Extremes are black and white, so make those the two
+  ;; choices. Convert the color to grayscale (0.299R + 0.587G +
+  ;; 0.114B) and then choose either black or white depending upon
+  ;; which one is furthest away from the grayscale color calculated.
+  ;;
+  ;; Found via research? Tested?  Haha.
+  (let* ((rgb (font-color-rgb-components color))
+	 (r (nth 0 rgb))
+	 (g (nth 1 rgb))
+	 (b (nth 2 rgb))
+	 (grayscale nil)
+	 (white nil)
+	 (max nil)
+	 (black nil))
+    (if (and (<= r 255) (<= g 255) (<= b 255))
+	(setq max 255)
+      (setq max 65535))
+    (setq grayscale (+ (* 0.299 r) (* 0.587 g) (* 0.114 b))
+	  black grayscale
+	  white (- max grayscale))
+    (if (> white black)
+	"white"
+      "black")))
+
+;; Test function for XEmacs and the color inference code.
+;; (defun w3-display-test-inference (color)
+;;  (interactive "sColor: ")
+;;  (set-face-background 'default color (current-buffer))
+;;  (set-face-foreground 'default (w3-display-infer-contrasting-color color) (current-buffer)))
+
 (defun w3-make-face-emacs19 (name &optional doc-string temporary)
   "Defines and returns a new FACE described by DOC-STRING.
 If the face already exists, it is unmodified."
@@ -279,10 +346,11 @@ If the face already exists, it is unmodified."
 			    (car w3-face-color)
 			    (car w3-face-background-color))
 	w3-face-face (cdr-safe (assoc w3-face-descr w3-face-cache)))
-  (if (or w3-face-face (not (or (car w3-face-color)
-				(car w3-face-background-image)
-				(car w3-face-background-color)
-				w3-face-font-spec)))
+  (if (or w3-face-face
+	  (not (or (car w3-face-color)
+		   (car w3-face-background-image)
+		   (car w3-face-background-color)
+		   w3-face-font-spec)))
       nil				; Do nothing, we got it already
     (setq w3-face-face
 	  (w3-make-face (intern (format "w3-style-face-%05d" w3-face-index))
@@ -297,9 +365,10 @@ If the face already exists, it is unmodified."
 	(font-set-face-foreground w3-face-face (car w3-face-color)))
     (if (car w3-face-background-color)
 	(font-set-face-background w3-face-face (car w3-face-background-color)))
-    (setq w3-face-cache (cons
-			 (cons w3-face-descr w3-face-face)
-			 w3-face-cache)))
+    (if (not (car w3-face-background-image))
+	(setq w3-face-cache (cons
+			     (cons w3-face-descr w3-face-face)
+			     w3-face-cache))))
   w3-face-face)
 
 (defun w3-normalize-spaces (string)
@@ -407,6 +476,8 @@ If the face already exists, it is unmodified."
 			   (list 'personality (car w3-active-voices))))
   )
 
+(require 'mule-sysdp)
+
 (defun w3-display-get-cookie (args)
   (if (not (fboundp 'cookie))
       "Sorry, no cookies today."
@@ -419,31 +490,43 @@ If the face already exists, it is unmodified."
 	  (save-excursion
 	    (set-buffer (generate-new-buffer " *cookie*"))
 	    (url-insert-file-contents href)
-	    (write-region (point-min) (point-max) fname 5)
+ 	    (setq buffer-file-name nil)
+ 	    (set-buffer-modified-p nil)    
+	    (mule-write-region-no-coding-system (point-min) (point-max) fname 5)
 	    (setq w3-cookie-cache (cons (cons href fname) w3-cookie-cache))))
       (cookie fname st nd))))
 
+(defun w3-widget-buffer (widget)
+  (let ((overlay (or (widget-get widget :button-overlay)
+		     (widget-get widget :field-overlay)))
+	(extent (or (widget-get widget :button-extent)
+		    (widget-get widget :field-extent))))
+    (or (and overlay (overlay-buffer overlay))
+	(and extent (extent-buffer extent)))))
+
 (defun w3-widget-echo (widget &rest ignore)
-  (let* ((url (widget-get widget :href))
-	 (name (widget-get widget :name))
-	 (text (buffer-substring-no-properties (widget-get widget :from)
-					       (widget-get widget :to)))
-	 (title (widget-get widget :title))
-	 (check w3-echo-link)
-	 (msg nil))
-    (if url
-	(setq url (url-truncate-url-for-viewing url)))
-    (if name
-	(setq name (concat "anchor:" name)))
-    (if (not (listp check))
-	(setq check (cons check '(title url text name))))
-    (catch 'exit
-      (while check
-	(and (boundp (car check))
-	     (stringp (symbol-value (car check)))
-	     (> (length (symbol-value (car check))) 0)
-	     (throw 'exit (symbol-value (car check))))
-	(pop check)))))
+  (save-excursion
+    (set-buffer (or (w3-widget-buffer widget) (current-buffer)))
+    (let* ((url (widget-get widget :href))
+	   (name (widget-get widget :name))
+	   (text (buffer-substring-no-properties (widget-get widget :from)
+						 (widget-get widget :to)))
+	   (title (widget-get widget :title))
+	   (check w3-echo-link)
+	   (msg nil))
+      (if url
+	  (setq url (url-truncate-url-for-viewing url)))
+      (if name
+	  (setq name (concat "anchor:" name)))
+      (if (not (listp check))
+	  (setq check (cons check '(title url text name))))
+      (catch 'exit
+	(while check
+	  (and (boundp (car check))
+	       (stringp (symbol-value (car check)))
+	       (> (length (symbol-value (car check))) 0)
+	       (throw 'exit (symbol-value (car check))))
+	  (pop check))))))
 
 (defun w3-follow-hyperlink (widget &rest ignore)
   (let* ((target (or (widget-get widget :target) w3-base-target))
@@ -777,6 +860,7 @@ If the face already exists, it is unmodified."
 (defun w3-finalize-image-download (url buffer &optional widget face)
   (let ((glyph nil)
 	(node nil))
+    (url-uncompress)
     (message "Enhancing image...")
     (setq glyph (image-normalize (cdr-safe (assoc url-current-mime-type
 						  w3-image-mappings))
@@ -794,7 +878,7 @@ If the face already exists, it is unmodified."
 	  (erase-buffer)
 	  (insert (aref glyph 2))
 	  (setq glyph temp-fname)
-	  (write-region (point-min) (point-max) temp-fname)
+	  (mule-write-region-no-coding-system (point-min) (point-max) temp-fname)
 	  (kill-buffer (current-buffer)))
 	(setq glyph (make-glyph (list (cons 'x glyph))))
 	(condition-case ()
@@ -860,9 +944,7 @@ If the face already exists, it is unmodified."
 (defmacro w3-image-alt (src)
   (`
    (let* ((doc-alt (w3-get-attribute 'alt))
-	  (alt (or (and doc-alt
-			(not (string-equal doc-alt ""))
-			doc-alt)
+	  (alt (or doc-alt
 		   (cond
 		    ((null w3-auto-image-alt) "")
 		    ((eq t w3-auto-image-alt)
@@ -1648,10 +1730,14 @@ Can sometimes make the structure of a document clearer")
 		 (cond
 		  ((= (aref prev-rowspans i) 0)
 		   ;; First row, insert the top horizontal divider
-		   (if border
+;;; Everything in this functino commented out with ;;; is done so to
+;;; borderless tables work better.  This was an attempt to not show
+;;; the 'spaces' border around the table, to save screen real estate,
+;;; but it messes up indentation on cell columns.
+;;;		   (if border
 		       (w3-insert-terminal-char
 			(w3-table-lookup-char t nil t nil border-char) 
-			(aref column-dimensions i)))
+			(aref column-dimensions i));;;)
 		   (setq i (1+ i)))
 		  ((car (aref formatted-cols i))
 		   ;; Slap in the rows
@@ -1665,11 +1751,11 @@ Can sometimes make the structure of a document clearer")
 		   (setq lflag nil)
 		   (setq i (+ i (max (aref table-colspans i)
 				     (aref prev-colspans  i) 1))))))
-	       (if (not border)
-		   nil
+;;;	       (if (not border)
+;;;		   nil
 		 (w3-insert-terminal-char
 		  (w3-table-lookup-char lflag (/= row 1) nil t border-char))
-		 (insert "\n")))
+		 (insert "\n"));;;)
 	     
 	     ;; recalculate height (in case we've shortened a rowspanning cell)
 	     (setq height 0 
@@ -2276,6 +2362,32 @@ Format: (((image-alt row column) . offset) ...)")
 		     (progn
 		       (setq fore (w3-fix-color (w3-get-attribute 'text)))
 		       (setf (car w3-face-color) fore)))
+
+		 ;; Here we do some sanity checking of the colors
+		 ;; selected by the author.
+
+		 ;; If they specify the foreground and not the
+		 ;; background, _AND_ we determine the contrast is not
+		 ;; enough, then don't honor the foreground at all.
+		 (if (and fore (not back) (not pixm) (w3-display-foreground-useless-p fore))
+		     (setq fore nil))
+
+		 ;; If we wanted to be really weird, we could infer
+		 ;; the background for them instead of just ignoring
+		 ;; the foreground.  But I think this might be too
+		 ;; shocking for the average user. :)
+		 ;; 
+		 ;; (setq back (w3-display-infer-contrasting-color fore)))
+
+		 ;; If they specify the background and not the
+		 ;; foreground, _AND_ we determine the contrast is not
+		 ;; enough, then infer a new foreground color.
+		 (if (and back (not pixm) (not fore) (w3-display-background-useless-p back))
+		     (setq fore (w3-display-infer-contrasting-color back)))
+
+		 (setf (car w3-face-color) fore)
+		 (setf (car w3-face-background-color) back)
+
 		 (if (not font-running-xemacs)
 		     (setq w3-display-background-properties (cons fore back))
 		   (if pixm
@@ -2286,7 +2398,7 @@ Format: (((image-alt row column) . offset) ...)")
 		       (font-set-face-background 'default back (current-buffer)))))
 	       (w3-handle-content node)))
 	    (*document
-	     (let ((info (mapcar (lambda (x) (cons x (symbol-value x)))
+	     (let ((info (mapcar (lambda (x) (cons x (and (boundp x) (symbol-value x))))
 				 w3-persistent-variables)))
 	       (if (not w3-display-same-buffer)
 		   (set-buffer (generate-new-buffer "Untitled")))
@@ -2303,7 +2415,9 @@ Format: (((image-alt row column) . offset) ...)")
 		 (error (message  "W3 buffer %s is being drawn." (buffer-name (current-buffer)))))
 
 	       (buffer-disable-undo (current-buffer))
-	       (mapcar (function (lambda (x) (set (car x) (cdr x)))) info)
+	       (mapcar (function (lambda (x)
+				   (if (boundp (car x))
+				       (set (car x) (cdr x))))) info)
 	       ;; ACK!  We don't like filladapt mode!
 	       (set (make-local-variable 'filladapt-mode) nil)
 	       (set (make-local-variable 'adaptive-fill-mode) nil)
@@ -2556,7 +2670,10 @@ Format: (((image-alt row column) . offset) ...)")
        (w3-fixup-eol-faces))
   (message "Drawing... done"))
 
+;;;###autoload
 (defun w3-region (st nd)
+  "Parse and display the region of this buffer between ST and ND."
+  (interactive "r")
   (if (not w3-setup-done) (w3-do-setup))
   (let* ((source (buffer-substring st nd))
 	 (w3-display-same-buffer t)
@@ -2595,13 +2712,14 @@ Format: (((image-alt row column) . offset) ...)")
 	 (source-buf (current-buffer))
 	 (parse (w3-parse-buffer source-buf)))
     (set-buffer-modified-p nil)
-    (w3-draw-tree parse)
-    (kill-buffer source-buf)
-    (set-buffer-modified-p nil)
-    (setq w3-current-source source
-	  w3-current-parse parse)
-    (w3-finish-drawing)
-    (w3-mode)
+    (unwind-protect
+	(w3-draw-tree parse)
+      (kill-buffer source-buf)
+      (set-buffer-modified-p nil)
+      (setq w3-current-source source
+	    w3-current-parse parse)
+      (w3-finish-drawing)
+      (w3-mode))
     (set-buffer-modified-p nil)
     (if url-keep-history
 	(let ((url (url-view-url t)))
