@@ -1,12 +1,12 @@
 ;;; w3-menu.el --- Menu functions for emacs-w3
 ;; Author: wmperry
-;; Created: 1998/06/12 16:05:26
-;; Version: 1.52
+;; Created: 1998/12/23 19:34:05
+;; Version: 1.2
 ;; Keywords: menu, hypermedia
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Copyright (c) 1996 by William M. Perry <wmperry@cs.indiana.edu>
-;;; Copyright (c) 1996 - 1998 Free Software Foundation, Inc.
+;;; Copyright (c) 1996 - 1999 Free Software Foundation, Inc.
 ;;;
 ;;; This file is part of GNU Emacs.
 ;;;
@@ -364,6 +364,10 @@ on that platform."
    ["Rename Bookmark" w3-hotlist-rename-entry t]
    ["Append Bookmark List" w3-hotlist-append t]
    "----"
+   ["Add Keyword" w3-hotindex-add-key t]
+   ["Remove Keyword" w3-hotindex-rm-key t]
+   ["Query Keyword" w3-hotindex-query t]
+   "----"
    (if w3-menu-filters-supported-p
        '("Bookmarks" :filter w3-menu-hotlist-constructor)
      ["Bookmarks" w3-menu-e19-show-hotlist-menu t])
@@ -420,6 +424,9 @@ on that platform."
    ["Honor Color Requests" (setq w3-user-colors-take-precedence
 				 (not w3-user-colors-take-precedence))
     :style toggle :selected (not w3-user-colors-take-precedence)]
+   ["Honor Font Requests" (setq w3-user-fonts-take-precedence
+				 (not w3-user-fonts-take-precedence))
+    :style toggle :selected (not w3-user-fonts-take-precedence)]
    "---"
    ["Reload Stylesheets" w3-refresh-stylesheets t]
    )
@@ -505,6 +512,7 @@ on that platform."
 	(define-key map [rootmenu w3 edit] 'undefined)
 	(define-key map [rootmenu w3 options] 'undefined)
 	(define-key map [rootmenu w3 buffer] 'undefined)
+	(define-key map [rootmenu w3 mule] 'undefined)
 	(define-key map [rootmenu w3 tools] 'undefined)
 	(define-key map [rootmenu w3 help] 'undefined)
 	(define-key map [rootmenu w3 help-menu] 'undefined)
