@@ -1,12 +1,12 @@
 ;;; w3-vars.el,v --- All variable definitions for emacs-w3
 ;; Author: wmperry
-;; Created: 1998/04/17 19:06:08
-;; Version: 1.157
+;; Created: 1999/01/07 15:10:36
+;; Version: 1.2
 ;; Keywords: comm, help, hypermedia
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Copyright (c) 1993 - 1996 by William M. Perry <wmperry@cs.indiana.edu>
-;;; Copyright (c) 1996 - 1998 Free Software Foundation, Inc.
+;;; Copyright (c) 1996 - 1999 Free Software Foundation, Inc.
 ;;;
 ;;; This file is part of GNU Emacs.
 ;;;
@@ -34,7 +34,7 @@
 (require 'wid-edit)			; For `widget-keymap'
 
 (defconst w3-version-number
-  (let ((x "p4.0pre.23"))
+  (let ((x "p4.0pre.39"))
     (if (string-match "State:[ \t\n]+.\\([^ \t\n]+\\)" x)
 	(setq x (substring x (match-beginning 1) (match-end 1)))
       (setq x (substring x 1)))
@@ -42,7 +42,7 @@
      (function (lambda (x) (if (= x ?-) "." (char-to-string x)))) x ""))
   "Version # of w3-mode.")
 
-(defconst w3-version-date (let ((x "1998/04/17 19:06:08"))
+(defconst w3-version-date (let ((x "1999/01/07 15:10:36"))
 			    (if (string-match "Date: \\([^ \t\n]+\\)" x)
 				(substring x (match-beginning 1) (match-end 1))
 			      x))
@@ -406,6 +406,7 @@ last character position that was correctly filled.")
 (defvar w3-continuation '(url-uncompress)
   "List of functions to call to process a document completely.")
 (defvar w3-current-isindex nil "Is the current document a searchable index?")
+(defvar w3-current-buffer nil "Is the current W3 buffer")
 (defvar w3-current-last-buffer nil "Last W3 buffer seen before this one.")
 (defvar w3-current-links nil "An assoc list of <link> tags for this doc.")
 (defvar w3-current-metainfo nil "An assoc list of <meta> tags for this doc.")
@@ -433,6 +434,20 @@ returns.")
 (defvar w3-submit-button nil
   "A widget object specifying what button was pressed to submit a form.")
 
+(defvar w3-meta-content-type-charset-regexp
+  (concat "<meta[ \t]+http-equiv=\"?Content-type\"?[ \t]+content=\"\\([^;]+\\)"
+	  ";[ \t]*charset=\"?\\([^\"]+\\)\"?"
+	  ">")
+  "Regexp used in parsing `<META HTTP-EQUIV=\"Content-Type\" content=\"...;charset=...\">
+for a charset indication")
+
+(defvar w3-meta-charset-content-type-regexp
+  (concat "<meta[ \t]+content=\"\\([^;]+\\)"
+	  ";[ \t]*charset=\"?\\([^\"]+\\)\"?"
+	  "[ \t]+http-equiv=\"?Content-type\"?>")
+  "Regexp used in parsing `<META content=\"...;charset=...\" HTTP-EQUIV=\"Content-Type\">
+for a charset indication")
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; buffer-local variables to keep around when going into w3-mode
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -471,6 +486,7 @@ returns.")
     w3-base-target
     w3-target-window-distances
     w3-frameset-structure
+    buffer-file-coding-system
     )
   "A list of variables that should be preserved when entering w3-mode.")
 
@@ -528,6 +544,11 @@ returns.")
 (define-key w3-mode-map "hI"       'w3-hotlist-add-document-at-point)
 (define-key w3-mode-map "hR"       'w3-hotlist-refresh)
 
+(define-key w3-mode-map "x" (make-sparse-keymap))
+(define-key w3-mode-map "xa" 'w3-hotindex-add-key)
+(define-key w3-mode-map "xd" 'w3-hotindex-rm-key)
+(define-key w3-mode-map "xq" 'w3-hotindex-query)
+
 (define-key w3-mode-map "HF"       'w3-history-forward)
 (define-key w3-mode-map "HB"       'w3-history-backward)
 (define-key w3-mode-map "Hv"       'w3-show-history-list)
@@ -549,6 +570,7 @@ returns.")
 (define-key w3-mode-map "U"        'w3-use-links)
 (define-key w3-mode-map "V"        'w3-view-this-url)
 (define-key w3-mode-map "\C-?"     'scroll-down)
+(define-key w3-mode-map [backspace] 'scroll-down)
 (define-key w3-mode-map "\C-c\C-b" 'w3-show-history-list)
 (define-key w3-mode-map "\C-c\C-v" 'w3-version)
 (define-key w3-mode-map "\C-o"     'w3-fetch)
@@ -586,6 +608,10 @@ returns.")
 (define-key w3-mode-map [(control alt t)] 'url-list-processes)
 (define-key w3-mode-map [(control meta t)] 'url-list-processes)
 
+;; Have fun with document ordering
+(define-key w3-mode-map [(meta space)] 'w3-next-document)
+(define-key w3-mode-map [(meta delete)] 'w3-prev-document)
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Keyword definitions
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
