@@ -1,12 +1,12 @@
 ;;; w3.el --- Main functions for emacs-w3 on all platforms/versions
 ;; Author: wmperry
-;; Created: 1998/06/22 23:32:54
-;; Version: 1.158
+;; Created: 1998/12/30 11:26:51
+;; Version: 1.6
 ;; Keywords: faces, help, comm, news, mail, processes, mouse, hypermedia
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Copyright (c) 1993 - 1996 by William M. Perry <wmperry@cs.indiana.edu>
-;;; Copyright (c) 1996 - 1998 Free Software Foundation, Inc.
+;;; Copyright (c) 1996 - 1999 Free Software Foundation, Inc.
 ;;;
 ;;; This file is part of GNU Emacs.
 ;;;
@@ -82,6 +82,7 @@
 
 (require 'cl)
 (require 'css)
+(require 'url-vars)
 (require 'w3-vars)
 (eval-and-compile
   (require 'w3-display))
@@ -237,6 +238,7 @@ See the variable `w3-notify' for the different notification behaviors."
 	    (buffer-enable-undo)
 	    (funcall view)
 	    (w3-notify-when-ready bufnam))
+	(setq w3-current-last-buffer w3-current-buffer)
 	(funcall view)))
      ((stringp view)
       (let ((fname (url-generate-unique-filename fmt))
@@ -439,6 +441,7 @@ With prefix argument, use the URL of the hyperlink under point instead."
    (t
     (let ((x (url-view-url t))
 	  (lastbuf (current-buffer))
+	  (w3-current-buffer (current-buffer))
 	  (buf (url-buffer-visiting url)))
       (if (or (not buf)
 	      (cond
@@ -470,8 +473,12 @@ With prefix argument, use the URL of the hyperlink under point instead."
 		    (setq w3-current-last-buffer lastbuf)))
 		 (t
 		  (w3-history-push x url)
-		  (w3-sentinel lastbuf)))))
-	(w3-history-push x (url-view-url t))
+		  (w3-sentinel lastbuf)))
+	      (w3-history-push x url)))
+	(save-excursion
+	  (set-buffer buf)
+	  (setq w3-current-last-buffer lastbuf))
+	(w3-history-push x url)
 	(if w3-track-last-buffer
 	    (setq w3-last-buffer buf))
 	(let ((w3-notify (if (memq w3-notify '(newframe bully 
@@ -895,16 +902,20 @@ and convert newlines into spaces."
 	       ((stringp under) under)
 	       (under (w3-view-this-url t))
 	       (t (url-view-url t))))
-	 (content-type "text/plain; charset=iso-8859-1")
+	 (content-charset (or (and (boundp 'buffer-file-coding-system)
+				   (symbol-value buffer-file-coding-system)
+				   (symbol-name buffer-file-coding-system))
+			      "iso-8859-1"))
+	 (content-type (concat "text/plain; charset=" content-charset))
 	 (str
 	  (save-excursion
 	    (cond
 	     ((and (equal "HTML Source" format) under)
-	      (setq content-type "text/html; charset=iso-8859-1")
+	      (setq content-type (concat "text/html; charset=" content-charset))
 	      (let ((url-source t))
 		(url-retrieve url)))
 	     ((equal "HTML Source" format)
-	      (setq content-type "text/html; charset=iso-8859-1")
+	      (setq content-type (concat "text/html; charset=" content-charset))
 	      (if w3-current-source
 		  (let ((x w3-current-source))
 		    (set-buffer (get-buffer-create url-working-buffer))
@@ -929,13 +940,13 @@ and convert newlines into spaces."
 		(ps-spool-buffer-with-faces)
 		(set-buffer ps-spool-buffer-name)))
 	     ((and under (equal "Formatted Text" format))
-	      (setq content-type "text/plain; charset=iso-8859-1")
+	      (setq content-type (concat "text/plain; charset=" content-charset))
 	      (w3-fetch url))
 	     ((equal "Formatted Text" format)
-	      (setq content-type "text/plain; charset=iso-8859-1"))
+	      (setq content-type (concat "text/plain; charset=" content-charset)))
 	     ((and under (equal "LaTeX Source" format))
 	      (let ((old-asynch (default-value 'url-be-asynchronous)))
-		(setq content-type "application/x-latex; charset=iso-8859-1")
+		(setq content-type (concat "application/x-latex; charset=" content-charset))
 		(unwind-protect
 		    (progn
 		      (setq-default url-be-asynchronous nil)
@@ -944,7 +955,7 @@ and convert newlines into spaces."
 		(w3-parse-tree-to-latex (w3-parse-buffer (current-buffer))
 					url)))
 	     ((equal "LaTeX Source" format)
-	      (setq content-type "application/x-latex; charset=iso-8859-1")
+	      (setq content-type (concat "application/x-latex; charset=" content-charset))
 	      (w3-parse-tree-to-latex w3-current-parse url)))
 	    (buffer-string))))
     (funcall url-mail-command)
@@ -1053,8 +1064,9 @@ ftp: reference"
       (find-file (url-filename url-current-object)))
      ((equal type "ftp")
       (find-file
-       (format "/%s@%s:%s"
-	       (url-user url-current-object)
+       (format "/%s%s:%s"
+	       (if (url-user url-current-object)
+		   (concat (url-user url-current-object) "@"))
 	       (url-host url-current-object)
 	       (url-filename url-current-object))))
      (t (message "Sorry, I can't get that file so you can alter it.")))))
@@ -1145,37 +1157,119 @@ ftp.w3.org:/pub/www/doc."
   '("text/plain" "text/html")
   "List of MIME types that require Mules' code conversion.")
 
-(defun w3-convert-code-for-mule (mmtype)
-  "Convert current data into the appropriate coding system"
+(defvar w3-compression-encodings
+  '("x-gzip" "gzip" "x-compress" "compress")
+  "List of MIME encodings that denote compression")
+
+(defvar w3-no-conversion-encodings
+  w3-compression-encodings
+  "List of MIME encodings that require Mule not to convert
+even though the MIME type is nil or listed in `w3-mime-list-for-code-conversion'.")
+
+(defvar w3-explicit-coding-system nil
+  "Coding system to decode documents.
+
+The global value is usually nil.  It will be bound locally if a user
+invokes some commands which read a coding system from the user.")
+
+(defun w3-convert-code-for-mule (mmtype mmcharset mmencoding)
+  "Decode current data by an appropriate coding system."
   (and (or (not mmtype)
 	   (member mmtype w3-mime-list-for-code-conversion))
-       (mule-code-convert-region
-	(point-min) (point-max)
-	(mule-detect-coding-version (point-min) (point-max)))))
+       (not (member mmencoding w3-no-conversion-encodings))
+       (let (charset-symbol coding-system)
+	 (cond (;; explicit coding system ? (through C-u [w3-reload-document])
+		(and w3-explicit-coding-system
+		     (mule-coding-system-p w3-explicit-coding-system))
+		(setq coding-system w3-explicit-coding-system))
+	       (;; explicit charset ? (through MIME headers or META tag)
+		(and (stringp mmcharset)
+		     (setq coding-system (w3-coding-system-for-mime-charset mmcharset))))
+	       ;; forced conversion ? (through user option w3-force-conversion-alist)
+	       ((and (url-filename url-current-object)
+		     (let ((force-alist w3-force-conversion-alist)
+			   (url (concat (url-host url-current-object)
+					(url-filename url-current-object))))
+		       (while (and force-alist (not coding-system))
+			 (if (string-match (car (car force-alist)) url)
+			     (setq coding-system (cdr (car force-alist))))
+			 (setq force-alist (cdr force-alist)))
+		       coding-system))
+		coding-system)
+	       (t;; otherwise try to detect coding-system
+		(setq coding-system
+		      (mule-detect-coding-version (url-host url-current-object)
+						  (point-min) (point-max)))))
+	 (mule-code-convert-region coding-system)
+	 (if (mule-coding-system-with-invalid-chars coding-system)
+	     (w3-replace-invalid-chars)))))
+
+(defun w3-coding-system-for-mime-charset (mmcharset)
+  (let ((coding-system nil)
+	(l w3-mime-charset-coding-alist))
+    (while l
+      (if (string-match (car (car l)) mmcharset)
+	  (setq coding-system (if (mule-coding-system-p (cdr (car l)))
+				  (cdr (car l)))
+		l nil)
+	(setq l (cdr l))))
+    coding-system))
+
+(defun w3-replace-invalid-chars ()
+  (let ((invalid-char-alist w3-invalid-sgml-char-replacement))
+    (while invalid-char-alist
+      (goto-char (point-min))
+      (if (< (caar invalid-char-alist) 256)
+	  (let ((str (char-to-string (caar invalid-char-alist)))
+		(repl (cdar invalid-char-alist)))
+	    (while (search-forward str nil t)
+	      (replace-match repl nil t))))
+      (setq invalid-char-alist (cdr invalid-char-alist)))))
 
 (defun w3-sentinel (&optional proc string)
-  (set-buffer url-working-buffer)
-  (if (or (stringp proc)
-	  (bufferp proc)) (setq w3-current-last-buffer proc))
-  (remove-hook 'after-change-functions 'url-after-change-function)
-  (if url-be-asynchronous
-      (progn
-	(cond
-	 ((not (get-buffer url-working-buffer)) nil)
-	 ((url-mime-response-p) (url-parse-mime-headers)))
-	(if (not url-current-mime-type)
-	    (setq url-current-mime-type (or (mm-extension-to-mime
-					     (url-file-extension
-					      (url-filename
-					       url-current-object)))
-					    "text/html")))))
-  (if (not (string-match "^www:" (or (url-view-url t) "")))
-      (w3-convert-code-for-mule url-current-mime-type))
-      
-  (let ((x (w3-build-continuation))
-	(url (url-view-url t)))
-    (while x
-      (funcall (pop x)))))
+  (let ((minibuffer-window (active-minibuffer-window)))
+    (unwind-protect
+	(progn
+	  (if minibuffer-window
+	      (other-window 1))
+	  (set-buffer url-working-buffer)
+	  (if (or (stringp proc)
+		  (bufferp proc)) (setq w3-current-last-buffer proc))
+	  (remove-hook 'after-change-functions 'url-after-change-function)
+	  (if url-be-asynchronous
+	      (progn
+		(cond
+		 ((not (get-buffer url-working-buffer)) nil)
+		 ((url-mime-response-p) (url-parse-mime-headers)))
+		(if (not url-current-mime-type)
+		    (setq url-current-mime-type (or (mm-extension-to-mime
+						     (url-file-extension
+						      (url-filename
+						       url-current-object)))
+						    "text/html")))))
+	  ;; hack for charset not indicated in MIME headers but in a META tag ...
+	  (if (not url-current-mime-charset)
+	      (save-excursion
+		(goto-char (point-min))
+		(if (or (re-search-forward w3-meta-content-type-charset-regexp nil t)
+			(re-search-forward w3-meta-charset-content-type-regexp nil t))
+		    (setq url-current-mime-type
+			  (buffer-substring-no-properties (match-beginning 1) (match-end 1))
+			  url-current-mime-charset
+			  (buffer-substring-no-properties (match-beginning 2) (match-end 2))))))
+	  (if (not (string-match "^www:" (or (url-view-url t) "")))
+	      (w3-convert-code-for-mule (and (stringp url-current-mime-type)
+					     (downcase url-current-mime-type))
+					(and (stringp url-current-mime-charset)
+					     (downcase url-current-mime-charset))
+					(and (stringp url-current-mime-encoding)
+					     (downcase url-current-mime-encoding))))
+	  (let ((x (w3-build-continuation))
+		(url (url-view-url t)))
+	    (while x
+	      (funcall (pop x)))))
+      (if minibuffer-window
+	  (set-frame-selected-window (selected-frame) minibuffer-window)))))
 
 (defun w3-show-history-list ()
   "Format the url-history-list prettily and show it to the user"
@@ -1227,7 +1321,7 @@ ftp.w3.org:/pub/www/doc."
 	  (set-buffer ps-spool-buffer-name)))
        ((equal "LaTeX Source" format)
 	(w3-parse-tree-to-latex w3-current-parse url)))
-      (write-region (point-min) (point-max) fname))))
+      (mule-write-region-no-coding-system (point-min) (point-max) fname))))
 
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -1587,9 +1681,11 @@ displayed in a new buffer."
      (base
       (setq url-current-object base))      
      ((buffer-file-name buffer)
-      (setq url-current-object
-	    (url-generic-parse-url (concat "file:"
-					   (buffer-file-name buffer)))))
+      (let ((path (buffer-file-name buffer)))
+	(if (string-match "^//" path)
+	    (setq path (concat "/%2f" (substring path 2))))
+	(setq url-current-object
+	      (url-generic-parse-url (concat "file:" path)))))
      (t
       (setq url-current-object
 	    (url-generic-parse-url "file:/")
@@ -1670,15 +1766,19 @@ cached and in local mode."
   (let ((url-standalone-mode nil))
     (w3-reload-document)))
 
-(defun w3-reload-document ()
-  "Reload the current document"
-  (interactive)
+(defun w3-reload-document (&optional explicit-coding-system)
+  "Reload the current document.
+With prefix argument, it reads a coding system to decode the document."
+  (interactive
+   (list (if (and (featurep 'mule) current-prefix-arg)
+	     (read-coding-system "Coding system: "))))
   (let ((tmp (url-view-url t))
 	(pnt (point))
 	(window-start (progn
 			(move-to-window-line 0)
 			(point)))
-	(url-request-extra-headers '(("Pragma" . "no-cache"))))
+	(url-request-extra-headers '(("Pragma" . "no-cache")))
+	(w3-explicit-coding-system explicit-coding-system))
     (kill-buffer (current-buffer))
     (w3-fetch tmp)
     (goto-char pnt)
@@ -2151,16 +2251,10 @@ dumped with emacs."
   (if (and (get-buffer buff) (buffer-name buff))
       (save-excursion
 	(set-buffer buff)
-	(let ((require-final-newline nil)
-	      (crypt-encoding-alist nil)
-	      (jka-compr-compression-info-list nil)
-	      (jam-zcat-filename-list nil)
-	      (write-file-hooks nil)
-	      (write-contents-hooks nil))
-	  (mule-write-region-no-coding-system (point-min) (point-max) fname)
-	  (message "Download of %s complete." (url-view-url t))
-	  (sit-for 3)
-	  (kill-buffer buff)))))
+	(mule-write-region-no-coding-system (point-min) (point-max) fname)
+	(message "Download of %s complete." (url-view-url t))
+	(sit-for 3)
+	(kill-buffer buff))))
 
 (defun w3-download-url-at-point ()
   "Download the URL under point."
@@ -2240,6 +2334,40 @@ to disk."
      (t
       (w3-fetch href)))))
 
+;;;###autoload
+(defun w3-next-document ()
+  (interactive)
+  (let ((link (or (let ((rel (assq 'rel w3-current-links)))
+                    (and rel (assoc "next" rel)))
+                  (let ((rev (assq 'rev w3-current-links)))
+                    (and rev (or (assoc "previous" rev)
+                                 (assoc "prev" rev))))))
+        href)
+    (and link (setq link (cdr link)))
+    (while (and link (null href))
+      (setq href (plist-get (car link) 'href))
+      (setq link (cdr link)))
+    (if href
+        (w3-fetch href)
+      (error "No NEXT document"))))
+
+;;;###autoload
+(defun w3-prev-document ()
+  (interactive)
+  (let ((link (or (let ((rel (assq 'rel w3-current-links)))
+                    (and rel (or (assoc "previous" rel)
+                                 (assoc "prev" rel))))
+                  (let ((rev (assq 'rev w3-current-links)))
+                    (and rev (assoc "next" rev)))))
+        href)
+    (and link (setq link (cdr link)))
+    (while (and link (null href))
+      (setq href (plist-get (car link) 'href))
+      (setq link (cdr link)))
+    (if href
+        (w3-fetch href)
+      (error "No PREVIOUS document"))))
+
 (defun w3-widget-forward (arg)
   "Move point to the next field or button.
 With optional ARG, move across that many fields."
@@ -2353,13 +2481,14 @@ Current keymap is:
   (or w3-setup-done (w3-do-setup))
   (if (interactive-p)
       (w3-preview-this-buffer)
-    (let ((tmp (mapcar (function (lambda (x) (cons x (symbol-value x))))
+    (let ((tmp (mapcar (function (lambda (x) (cons x (and (boundp x) (symbol-value x)))))
 		       w3-persistent-variables)))
       ;; Oh gross, this kills buffer-local faces in XEmacs
       ;;(kill-all-local-variables)
       (use-local-map w3-mode-map)
       (setq mode-name "WWW")
-      (mapcar (function (lambda (x) (set-variable (car x) (cdr x)))) tmp)
+      (mapcar (function (lambda (x) (if (boundp (car x))
+					(set-variable (car x) (cdr x))))) tmp)
       (setq major-mode 'w3-mode)
       (w3-mode-version-specifics)
       (w3-menu-install-menus)
